import { permission, role, router } from 'decorators/router'
import BaseController from 'common/base.controller'
import { Admin, ADMINS } from 'models/admin/Admin'
import { AdminRole } from 'models/admin/AdminRole'
import { ZError } from 'common/ZError'
import { generate_captcha } from '../../utils/captcha.util'
import { isTrue } from '../../utils/string.util'
import { NonceRecord } from 'models/admin/NonceRecord'
import { buildLoginSignMsg, recoverTypedSignatureV4 } from 'utils/chain.util'
import { Dept } from 'models/admin/Dept'

const LOGIN_TIP = 'This signature is just to verify your identity'
export default class AccountController extends BaseController {
  @router('post /api/admin/login')
  async login(req, res) {
    const { username, password } = req.params
    if (!username || !password) {
      throw new ZError(10, 'no username or password')
    }
    let account = await Admin.findByName(username)
    if (!account) {
      throw new ZError(11, 'account not found')
    }
    if (account.locked) {
      throw new ZError(12, 'account locked')
    }
    if (!account.verifyPassword(password)) {
      throw new ZError(13, 'password error')
    }
    const token = await res.jwtSign({ id: account.id })
    return { token }
  }

  @router('get /api/admin/wallet_nonce')
  async walletNonce(req, res) {
    let record = new NonceRecord()
    await record.save()
    return { nonce: record.id, tips: LOGIN_TIP }
  }
  // wallet login
  @router('post /api/admin/wallet_login')
  async loginWallet(req, res) {
    const { nonce, signature, account } = req.params
    if (!nonce || !signature || !account) {
      throw new ZError(10, 'params mismatch')
    }
    let record = await NonceRecord.findById(nonce)
    if (!record || record.status !== 0) {
      throw new ZError(11, 'nonce invalid')
    }
    record.status = 1
    await record.save()
    const signObj = buildLoginSignMsg(nonce, LOGIN_TIP)
    const recover = recoverTypedSignatureV4(signObj, signature)
    console.log(recover)
    let accountData = await Admin.findByAddress(account)
    if (!accountData) {
      throw new ZError(11, 'account not found')
    }
    if (accountData.locked) {
      throw new ZError(12, 'account locked')
    }

    const token = await res.jwtSign({ id: accountData.id })
    return { token }
  }

  @router('post /api/admin/logout')
  async logout(req, res) {
    return {}
  }
  @permission(['admin:edit', 'shopadmin:edit'])
  @router('post /api/admin/save')
  async save(req) {
    const { id, username, password, roles, showname, sex, locked, department, level, avatar } = req.params
    let account
    if (id) {
      account = await Admin.findById(id)
      if (!account) {
        throw new ZError(11, 'account not found')
      }
    } else {
      account = await Admin.findByName(username)
      if (account) {
        throw new ZError(10, 'account already exists')
      }
      account = new Admin()
      account.level = 9
    }
    if (username && ADMINS.indexOf(username) == -1) {
      account.username = username
    }
    let admin = req.user
    if (admin.level > account.level) {
      throw new ZError(13, 'operate no permission')
    }
    account.roles = roles
    if (password) {
      account.updatePassword(password)
    }
    if (showname) account.showname = showname
    account.sex = sex || '0'
    // 管理员不需要设置部门属性
    if (ADMINS.indexOf(username) < 0) {
      if (admin.level > 1) {
        account.department = admin.department
      } else {
        account.department = department
      }
    }
    if (level) {
      account.level = Math.max(level, admin.level)
    }

    if (avatar) {
      account.avatar = avatar
    }
    if (locked != undefined) {
      if (locked) {
        if (ADMINS.indexOf(username) >= 0) {
          throw new ZError(20, 'can not lock admin account')
        }
        account.locked = true
        account.lockedTime = new Date()
      } else {
        account.locked = false
      }
    }
    await account.save()
    return account.toJson()
  }

  @permission(['admin:read'])
  @router('get /api/admins')
  async users(req) {
    const user = req.user
    let { dept, level } = req.params
    let queryData: any = { deleted: false }
    if (!user.isSysAdmin()) {
      queryData.show = true
    }
    if (dept) {
      queryData.department = dept
    }
    if (level) {
      queryData.level = level
    }
    let users = await Admin.find(queryData)
    return users.map(o => o.toJson())
  }

  @role('anon')
  @router('get /api/simple_admins')
  async simpleUsers(req) {
    const user = req.user
    let { dept, level } = req.params
    let queryData: any = { deleted: false }
    if (!user.isSysAdmin()) {
      queryData.show = true
    }
    if (dept) {
      queryData.department = dept
    }
    if (level) {
      queryData.level = level
    }
    let users = await Admin.find(queryData)
    return users.map(o => o.simpleInfo())
  }

  @permission('admin:read')
  @router('get /api/user/:uid/info')
  async info(req) {
    let uid = req.params.uid
    const account = await Admin.findById(uid)
    return { account: account, user: req.user }
  }

  @permission('self:read')
  @router('post /api/admin/info')
  async myInfo(req) {
    let account = req.user
    let roleObjs = await AdminRole.find({ _id: { $in: account.roles } })
    let permissionSet = new Set()
    for (let role of roleObjs) {
      role.permissions.forEach(permissionSet.add, permissionSet)
    }
    permissionSet.add('self:*')
    let result: any = account.toJson()
    result.permissions = [...permissionSet]
    if (account.department) {
      const record = await Dept.findById(account.department)
      if (record) {
        result.deptname = record.name
      }
    }
    result.flowRouter = []
    return result
  }

  @permission('self:edit')
  @router('post /api/admin/passwd')
  async changePass(req) {
    let { passwordOld, passwordNew } = req.params
    const account = req.user
    if (!account) {
      throw new ZError(10, 'account not found')
    }
    if (!account.verifyPassword(passwordOld)) {
      throw new ZError(11, 'old password error')
    }
    account.updatePassword(passwordNew)
    await account.save()
    return account.toJson()
  }

  @permission('self:edit')
  @router('post /api/admin/update_info')
  async changeInfo(req) {
    let { showname, sex, avatar } = req.params
    const account = req.user
    if (!account) {
      throw new ZError(10, 'account not found')
    }
    account.showname = showname
    account.sex = sex
    account.avatar = avatar
    await account.save()
    return account.toJson()
  }

  @permission(['admin:lock', 'shopadmin:lock'])
  @router('post /api/admin/:uid/locker')
  async changeLocked(req) {
    let { uid, lock } = req.params
    const account = await Admin.findById(uid)
    if (!account) {
      throw new ZError(10, 'account not found')
    }
    let admin = req.user
    const locker = isTrue(lock)
    if (admin.id === account.id && locker) {
      throw new ZError(14, 'can not lock self')
    }
    if (admin.level > 1) {
      if (admin.department !== account.department || account.level < admin.level) {
        throw new ZError(13, 'operate no permission')
      }
    }

    account.locked = isTrue(locker)
    if (locker) {
      account.lockTime = new Date()
    }
    await account.save()
    return account.toJson()
  }

  @permission(['admin:delete', 'shopadmin:delete'])
  @router('post /api/admin/:uid/delete')
  async deleteAdmin(req: any) {
    let { uid } = req.params
    if (!uid) {
      throw new ZError(10, 'param error')
    }
    const account = await Admin.findById(uid)
    if (!account) {
      throw new ZError(11, 'account not found')
    }
    if (ADMINS.indexOf(account.username) >= 0) {
      throw new ZError(12, 'can`t delete admin')
    }
    let admin = req.user
    if (admin.id === account.id) {
      throw new ZError(14, 'can not delete self')
    }
    if (admin.level > 1) {
      if (admin.department !== account.department || account.level < admin.level) {
        throw new ZError(13, 'operate no permission')
      }
    }
    account.deleted = true
    account.deleteTime = new Date()
    await account.save()
    return {}
  }

  @role('anon')
  @router('get /api/login/captcha')
  async getCaptcha(req: any, res: any) {
    const cap = generate_captcha()
    // req.session.captcha_admin_login = cap.text;
    // req.session.captcha_admin_login_stamp = Date.now();
    res.type('image/svg+xml')
    res.send(cap.data)
  }
}
