import BaseController from 'common/base.controller'
import { ZError } from 'common/ZError'
import { permission, role, router } from 'decorators/router'
import GlobalEvent, { FLOW_END } from 'events/GlobalEvent'
import { IAction } from 'flowactions/base/IAction'
import { FlowDefinition } from 'models/flow/FlowDefinition'
import { FlowForm } from 'models/flow/FlowForm'
import { FlowInstance, FlowStatus, FlowTask } from 'models/flow/FlowInstance'
import { FlowNode, NodeType } from 'models/flow/FlowNode'

export default class FlowInstanceController extends BaseController {
  @permission(['instance:edit', 'flow:exec'])
  @router('post /api/flow/instance')
  async save(req) {
    let { _id } = req.params
    let user = req.user
    let record
    if (!_id) {
      record = new FlowInstance(req.params)
      record.createdBy = user.id
    } else {
      record = await FlowInstance.insertOrUpdate({ _id: _id }, {})
      record.updateFromReq(req.params)
    }
    await record.save()
    return record.toJson()
  }

  @permission(['instance:read', 'flow:exec'])
  @router('get /api/flow/instance/:key')
  async info(req) {
    const { key } = req.params
    const record = await FlowInstance.findById(key)
    return record.toJson()
  }

  @permission(['instance:read', 'flow:exec'])
  @router('post /api/flow/instances')
  async list(req) {
    const records = await FlowInstance.pageQuery(req.params)
    return records
  }
  /**
   * 1. 我参与了的流程 {'tasks.executor': user.id}
   * 2. 我可以参与的流程: $and: [{ 'tasks.executors': user.id }, { 'tasks.status': 1 }]
   */
  @permission(['instance:read', 'flow:exec'])
  @router('post /api/flow/myinstlist')
  async mylist(req) {
    let user = req.user
    let data = req.params
    let { start, limit, page } = data
    limit = +limit || 10
    start = +start || (+page - 1) * limit || 0
    // @ts-ignore
    let { opt, sort } = FlowInstance.parseQueryParam(data)
    opt = Object.assign(opt, {
      $or: [
        { 'tasks.executor': user.id },
        {
          $and: [{ 'tasks.executors': user.id }, { 'tasks.status': 1 }],
        },
      ],
    })
    let records = await FlowInstance.find(opt).sort(sort).skip(start).limit(limit)
    let total = await FlowInstance.countDocuments(opt)
    return { records, total, start, limit }
  }

  @role('anon')
  @router('post /api/flow/client/result')
  async clientReport(req) {
    let { id, user, result, comment } = req.params
    if (!id) {
      throw new ZError(21, 'params mismatch')
    }
    let record = await FlowInstance.findById(id)
    if (!record) {
      throw new ZError(22, 'task not found')
    }
    await record.updateTaskStatus(user, result, comment)
    let currentTask = record.currentTask()
    if (currentTask?.nodeType === NodeType.end) {
      record.status = result == 2 ? FlowStatus.SUCCESS : FlowStatus.ERROR
      const now = Date.now()
      currentTask.endTime = now
      currentTask.status = result == 2 ? FlowStatus.SUCCESS : FlowStatus.ERROR
      record.endTime = now
      record.duration = now - record.startTime
      await record.save()
      new GlobalEvent().eventEmitter.emit(FLOW_END, this)
    }
    return record.toJson()
  }

  @permission(['instance:read', 'flow:exec'])
  @router('post /api/flow/instance/approval')
  async execute(req) {
    let { id, result, comment } = req.params
    if (!id) {
      throw new ZError(21, 'params mismatch')
    }
    let user = req.user
    let record = await FlowInstance.findById(id)
    if (!record) {
      throw new ZError(22, 'task not found')
    }
    await record.updateTaskStatus(user.id, result, comment)
    let currentTask = record.currentTask()
    if (currentTask?.nodeType === NodeType.exec && !!currentTask?.action) {
      let file = `../../flowactions/${currentTask.action}`
      let task: IAction = require(file)
      setImmediate(function () {
        //@ts-ignore
        new task.default().execute(record._id)
      })
    }
    return record.toJson()
  }

  @permission('instance:delete')
  @router('delete /api/flow/instance')
  async delete(req) {
    let { ids } = req.params
    if (!ids) {
      throw new ZError(11, 'params mismatch')
    }
    let result = await FlowInstance.deleteVirtual(ids)
    return result
  }

  @permission(['instance:update', 'flow:exec'])
  @router('post /api/flow/pre_instance')
  async fromDifinition(req) {
    let user = req.user
    let { did } = req.params
    const record = await FlowDefinition.findById(did)
    if (!record) {
      throw new ZError(10, 'flow definition not found')
    }
    let inst = new FlowInstance()
    inst.defId = did
    inst.category = record.category
    inst.name = record.name
    inst.startUser = user.id
    inst.newrecord = true
    let tasks: FlowTask[] = []
    let i = 0
    for (let sub of record.nodes) {
      let node = await FlowNode.findById(sub.node)
      let task = FlowInstance.taskFromNode(sub, node)
      if (i === 0) {
        i += 1
        task.startTime = Date.now()
        task.status = FlowStatus.ONGOING
        inst.currentId = task.taskId
      }
      tasks.push(task)
    }
    inst.tasks = tasks
    if (record.form) {
      let inputs: any[] = []
      let form = await FlowForm.findById(record.form)
      if (form) {
        for (let sub of form.nodes) {
          inputs.push(sub.toJson())
        }
      }
      inst.inputs = inputs
    }
    return inst.toJson()
  }
}
