import { getModelForClass, index, modelOptions, pre, prop, ReturnModelType } from '@typegoose/typegoose'
import { dbconn } from 'decorators/dbconn'

import { Base, TimeStamps } from '@typegoose/typegoose/lib/defaultClasses'

import { genRandomString, sha512 } from 'utils/security.util'
import { BaseModule } from '../Base'
import { formatAddress } from 'utils/chain.util'

/**
 * 生成密码的salt和hash
 * @param userpassword
 * @return {{salt: any, passwordHash: string}}
 */
export function saltHashPassword(userpassword: string) {
  let salt = genRandomString(16)
  return sha512(userpassword, salt)
}

export const ADMINS = ['admin', 'sysadmin']
/**
 * 验证密码
 * @param userpassword
 * @param passwordDb
 * @param salt
 * @return {boolean}
 */
export function verifyPass(userpassword: string, passwordDb: string, salt: string) {
  let passwordData = sha512(userpassword, salt)
  return passwordData.passwordHash === passwordDb
}

// @ts-ignore
export interface AdminClass extends Base, TimeStamps {}

@dbconn()
@index({ username: 1 }, { unique: true })
@index({ address: 1 }, { partialFilterExpression: { address: { $exists: true } } })
@modelOptions({ schemaOptions: { collection: 'admin_user', timestamps: true } })
@pre<AdminClass>('save', function () {
  this.newRecord = false
})
// @ts-ignore
class AdminClass extends BaseModule {
  @prop({ maxlength: 20, minlength: 5, required: true })
  public username: string
  @prop({ required: true })
  public password: string
  @prop()
  public address: string
  // 0: address 1: normal
  @prop({ required: true, default: 0 })
  public type: number
  @prop()
  public salt: string
  @prop()
  public showname?: string
  @prop({ default: 0 })
  public sex?: string
  @prop()
  public avatar?: string
  @prop({ type: () => [String] })
  public roles: string[]
  @prop()
  public createdBy: string
  @prop()
  public comment?: string
  @prop()
  public introduction?: string
  @prop({ default: true })
  public show: boolean
  /**
   * 删除标记
   */
  @prop({ default: false })
  public deleted: boolean
  /**
   * 删除时间
   */
  @prop()
  public deleteTime: Date

  @prop({ default: false })
  public locked: boolean
  @prop()
  public lockTime: Date
  /**
   * 部门
   * @type {string}
   */
  @prop()
  public department?: string
  /**
   * 等级
   * @type {number} 1: 系统级, 9: 部门专属帐号
   */
  @prop()
  public level: number
  /**
   * 是否为新记录, 只用于判断insertOrUpdate返回的记录是否为新纪录
   * @type {boolean}
   */
  @prop({ default: true })
  public newRecord: boolean

  public static async findByName(this: ReturnModelType<typeof AdminClass>, username) {
    return this.findOne({ username, deleted: false }).exec()
  }

  public static async findByAddress(this: ReturnModelType<typeof AdminClass>, address) {
    return this.findOne({ address, deleted: false }).exec()
  }

  /**
   * 是否是系统管理员
   * @return {boolean}
   */
  public isSysAdmin(): boolean {
    return !!this.roles.find(o => o === 'sysadmin')
  }
  public updatePassword(password) {
    if (password) {
      let passData = saltHashPassword(password)
      this.password = passData.passwordHash
      this.salt = passData.salt
    }
  }

  public verifyPassword(password) {
    return verifyPass(password, this.password, this.salt)
  }

  public simpleInfo() {
    return {
      id: this['_id'],
      username: this.username,
      showname: this.showname || this.username || formatAddress(this.address) || this['_id'],
    }
  }

  public toJson() {
    return {
      id: this['_id'],
      username: this.username,
      showname: this.showname,
      address: this.address,
      roles: this.roles,
      sex: this.sex,
      avatar: this.avatar,
      createdBy: this.createdBy,
      introduction: this.introduction,
      locked: this.locked,
      comment: this.comment,
      department: this.department,
      level: this.level,
    }
  }
}

export const Admin = getModelForClass(AdminClass, { existingConnection: AdminClass['db'] })
