import {
  getModelForClass,
  DocumentType,
  modelOptions,
  prop,
  ReturnModelType,
  Severity,
  mongoose,
} from '@typegoose/typegoose'
import { ObjectId } from 'bson'
import { dbconn } from 'decorators/dbconn'
import { noJson } from 'decorators/nojson'
import GlobalEvent, { FLOW_END, TASK_BEGIN } from 'events/GlobalEvent'

import { BaseModule } from '../Base'
import { NodeInfo } from './FlowDefinition'
import { InputInfo } from './FlowForm'
import { NodeType } from './FlowNode'

export enum FlowStatus {
  NOSTART = 0,
  ONGOING = 1,
  SUCCESS = 2,
  REVOKE = 3,
  PART_ERROR = 8,
  ERROR = 9,
}
/**
 * 流程实例的一个任务, 也可以认为是流程节点的实例
 * 一个流程实例包含一序列的任务
 */
@modelOptions({ schemaOptions: { _id: false } })
export class FlowTask {
  @prop()
  public taskId: string
  @prop()
  public nodeId: string
  @prop()
  public action: string
  @prop()
  public name: string
  @prop()
  public index: number
  @prop({ type: () => [String] })
  public executors: string[]
  @prop()
  public executor: string
  @prop({ enum: FlowStatus, default: FlowStatus.NOSTART })
  public status: FlowStatus
  @prop({ enum: NodeType, required: true })
  public nodeType: NodeType

  @prop()
  public startTime: number
  @prop()
  public endTime: number
  @prop()
  public duration: number
  @prop()
  public comment: string
}

/**
 * 一个流程实例
 */
@dbconn()
@modelOptions({
  schemaOptions: { collection: 'flow_instance', timestamps: true },
  options: { allowMixed: Severity.ALLOW },
})
class FlowInstanceClass extends BaseModule {
  @prop({ required: true })
  public name!: string
  // 流程定义id
  @prop({ required: true })
  public defId: string
  @prop({ required: true })
  public category: string
  @prop({ required: true, default: Date.now() })
  public startTime: number
  @prop()
  public endTime: number
  @prop()
  public duration: number
  @prop()
  public startUser: string
  @prop()
  public currentId: string
  @prop({ default: true })
  public newrecord: boolean
  // 0: 未开始; 1: 执行中; 2: 执行成功; 9: 执行失败;
  @prop({ enum: FlowStatus, default: FlowStatus.NOSTART })
  public status: FlowStatus

  @prop({ type: () => [FlowTask] })
  public tasks: FlowTask[]

  @prop({ type: () => [InputInfo] })
  public inputs: InputInfo[]

  @prop({ type: mongoose.Schema.Types.Mixed })
  public flowData: any

  @prop()
  public comment: string

  /**
   * 是否删除
   * @type {boolean}
   */
  @noJson()
  @prop({ default: false })
  public deleted: boolean
  @noJson()
  @prop()
  public deleteTime: Date

  public static async findByName(this: ReturnModelType<typeof FlowInstanceClass>, name: string) {
    return this.find({ name }).exec()
  }

  /**
   * 查找用户可操作的流程列表
   * @param this
   * @param user
   * @returns
   */
  public static async findMyFlow(this: ReturnModelType<typeof FlowInstanceClass>, user: string) {
    return this.find({
      status: FlowStatus.ONGOING,
      deleted: false,
      tasks: {
        $elemMatch: {
          status: FlowStatus.ONGOING,
          executors: user,
        },
      },
    })
  }

  public static taskFromNode(fNode: NodeInfo, node: any) {
    let task = new FlowTask()
    task.taskId = new ObjectId().toHexString()
    task.nodeId = fNode.node
    task.executors = fNode.executors
    task.name = node.name
    task.nodeType = node.type
    task.action = node.action
    return task
  }

  public currentTask(this: DocumentType<FlowInstanceClass>) {
    if (!this.currentId) {
      return null
    }
    return this.tasks.find(t => t.taskId === this.currentId)
  }
  /**
   * 审批
   * @param this
   * @param user
   * @param result
   */
  public async updateTaskStatus(
    this: DocumentType<FlowInstanceClass>,
    user: string,
    result: FlowStatus,
    comment: string,
  ) {
    if (!this.currentId) {
      throw new Error('task statue mismatch')
    }
    let task = this.tasks.find(t => t.taskId === this.currentId)
    if (!task) {
      throw new Error('task not exists')
    }
    if (!task.executors.includes(user)) {
      throw new Error('user has no permission')
    }
    if (task.status !== FlowStatus.ONGOING) {
      throw new Error('task status mismatch')
    }
    const now = Date.now()
    task.endTime = now
    task.duration = now - this.startTime
    task.executor = user
    task.status = result
    task.comment = comment
    if (result !== FlowStatus.SUCCESS) {
      this.status = FlowStatus.REVOKE
      await this.save()
      return
    }
    let index = this.tasks.findIndex(t => t.taskId === this.currentId)

    if (index >= this.tasks.length - 1) {
      this.endTime = now
      this.duration = now - this.startTime
      this.status = FlowStatus.SUCCESS
      new GlobalEvent().eventEmitter.emit(FLOW_END, this)
    } else {
      let nextTask = this.tasks[index + 1]
      nextTask.startTime = now
      nextTask.status = FlowStatus.ONGOING
      this.currentId = nextTask.taskId
      new GlobalEvent().eventEmitter.emit(TASK_BEGIN, nextTask)
    }
    await this.save()
  }
}

export const FlowInstance = getModelForClass(FlowInstanceClass, { existingConnection: FlowInstanceClass['db'] })
