import config from '../config/config'
import * as jetpack from 'fs-jetpack'
import { Game } from '../models/content/Game'
import { ZError } from '../common/ZError'
import { generateQr } from './Wechat'
import fs from 'fs'
import os from 'os'
import nodeHtmlToImage from 'node-html-to-image'
import axios from 'axios'
import * as http from 'http'
import { customAlphabet } from 'nanoid'
const nanoid = customAlphabet('0123456789abcdefghijklmnopqrstuvwxyz', 26)

export function generateUploadPath(subPath: string) {
  const base = config.file.upload_location
  const path = `${base}${subPath}`
  jetpack.dir(path)
  return path
}

/**
 * 生成包含该店铺信息的小程序码
 * @param {string} gameId
 * @param {string} version
 * @param {string} shop
 * @return {{file: string, url: string}}
 */
export async function generateQrFile({
  gameId,
  version,
  shop,
  params,
}: {
  gameId: string
  version: string
  shop: string
  params: string[]
}) {
  let subPath = `/qr/${gameId}/${version}`
  let paramStr = shop
  if (params && params.length > 0) {
    for (let _p of params) {
      subPath += `/${_p}`
      paramStr += `_${_p}`
    }
  }
  let path = generateUploadPath(subPath)

  let file = `${path}/${shop}.png`
  let url = `${config.file.show_url}${subPath}/${shop}.png`
  if (jetpack.exists(file) !== 'file') {
    let versionData = await Game.fetchVersionInfo(gameId, version)
    if (!versionData || !versionData.appid || !versionData.appsecret) {
      throw new ZError(20, 'game version not found')
    }
    if (paramStr.length > 30) {
      throw new ZError(21, '参数超过30个字符')
    }
    let result = await generateQr({
      appId: versionData.appid,
      appSecret: versionData.appsecret,
      scene: paramStr,
      file,
    })
    console.log(result)
  }
  return { file, url }
}

/**
 * 检查该游戏包含该店铺信息的小程序码是否存在
 * @param {string} gameId
 * @param {string} version
 * @param {string} shop
 * @return {boolean}
 */
export function checkQrExists(gameId: string, version: string, shop: string) {
  let subPath = `/qr/${gameId}/${version}`
  let path = generateUploadPath(subPath)
  let file = `${path}/${shop}.png`
  return jetpack.exists(file) === 'file'
}

/**
 * 生成优惠券的预览图
 * @param {string} shop
 * @param {string} couponId
 * @param {string} content
 * @return {Promise<string | Buffer | (string | Buffer)[]>}
 */
export function generateCouponImg(shop: string, couponId: string, content: string) {
  let subPath = `/coupon/${shop}`
  let path = generateUploadPath(subPath)
  let file = `${path}/${couponId}.png`
  if (content.search(/meta name=.+?viewport/) === -1) {
    const metaStr =
      '\n<meta name="viewport" content="width=device-width,initial-scale=1, minimum-scale=1, maximum-scale=1, user-scalable=no" />\n'
    content = content.replace('<head>', `<head>${metaStr}`)
  }
  return nodeHtmlToImage({
    output: file,
    html: content,
    transparent: false,
    puppeteerArgs: {
      args: ['--no-sandbox', '--disable-setuid-sandbox'],
      defaultViewport: {
        width: 375,
        height: 667,
        isMobile: true,
      },
    },
  })
}

export function getCouponUrl(shop: string, couponId: string) {
  let subPath = `/coupon/${shop}`
  return `${config.file.show_url}${subPath}/${couponId}.png`
}

export async function downloadRemoteFile(url: string) {
  const tempPath = os.tmpdir()
  const fileName = nanoid()
  const dest: string = `${tempPath}/${fileName}.png`
  if (jetpack.exists(dest)) {
    jetpack.remove(dest)
  }
  return new Promise((resolve, reject) => {
    axios({
      url,
      responseType: 'stream',
    }).then(resp => {
      const writer = fs.createWriteStream(dest)
      resp.data.pipe(writer)
      writer.on('finish', () => {
        console.log('finish')
        resolve && resolve(dest)
      })
      writer.on('error', err => {
        reject && reject(err)
      })
    })
  })
}
