import { RedisClient } from '../redis/RedisClient'
import { BaseConst } from '../constants/BaseConst'

const MAX_TIME = 5000000000000

function generateRankKey(subKey?: string) {
  if (subKey) {
    return `${BaseConst.RANK_SCORE}_${subKey}`
  } else {
    return BaseConst.RANK_SCORE
  }
}
/**
 * 更新排行榜数据
 * @param subKey
 * @param {string} accountId
 * @param {number} score
 * @return {Promise<void>}
 */
export async function updateRank(accountId: string, score: number, subKey?: string) {
  let scoreOld = await getAccountScore(accountId, subKey)
  let scoreL = parseFloat(`${score | 0}.${MAX_TIME - Date.now()}`)
  const key = generateRankKey(subKey)
  if (score > scoreOld) {
    await new RedisClient().zadd(key, scoreL, accountId)
  }
  const countTotal = (await new RedisClient().zcard(key)) as number
  const countOver = (await new RedisClient().zcount(key, 0, scoreL)) as number
  const over = countTotal ? countOver / countTotal : 0.99
  return { scoreReal: Math.max(score, scoreOld), over }
}

/**
 * 获取超越只
 * @param {string} accountId
 * @param {number} score
 * @param {string} subKey
 * @return {Promise<number>}
 */
export async function currentIdx(accountId: string, score: number, subKey?: string) {
  const key = generateRankKey(subKey)
  let countTotal = (await new RedisClient().zcard(key)) as number
  let scoreL = parseFloat(`${score | 0}.${MAX_TIME - Date.now()}`)
  let countOver = (await new RedisClient().zcount(key, 0, scoreL)) as number
  return countOver / countTotal
}

/**
 * 更新累计的排行榜
 * @param {string} accountId
 * @param {number} score
 * @param {string} subKey
 * @return {Promise<number>}
 */
export async function updateTotalRank(accountId: string, score: number, subKey?: string) {
  let scoreOld = await getAccountScore(accountId, subKey)
  score += scoreOld
  let scoreL = parseFloat(`${score | 0}.${MAX_TIME - Date.now()}`)
  await new RedisClient().zadd(generateRankKey(subKey), scoreL, accountId)
  return score
}

/**
 * 获取指定分数段的玩家
 * @param {number} min
 * @param {number} max
 * @param subKey
 * @return {Promise<unknown>}
 */
export async function usersByScore(min: number, max: number, subKey?: string) {
  return await new RedisClient().zrangebyscore(generateRankKey(subKey), min, max)
}

/**
 * 获取指定用户的排名
 * @param {string} accountId
 * @param subKey
 * @return {Promise<unknown>}
 */
export async function getAccountRank(accountId: string, subKey?: string) {
  return await new RedisClient().zrevrank(generateRankKey(subKey), accountId)
}
/**
 * 获取指定用户的积分
 * @param {string} accountId
 * @param subKey
 * @return {Promise<unknown>}
 */
export async function getAccountScore(accountId: string, subKey?: string) {
  let score = await new RedisClient().zscore(generateRankKey(subKey), accountId)
  if (score) {
    return +score << 0
  } else {
    return 0
  }
}

/**
 * 获取从start到end的玩家列表
 * @param {number} start
 * @param {number} end
 * @param subKey
 * @return {Promise<unknown>}
 */
export async function getRankList(start: number, end: number, subKey?: string) {
  return await new RedisClient().zrevrange(generateRankKey(subKey), start, end)
}

export async function getRankCount(subKey?: string) {
  return await new RedisClient().zcard(generateRankKey(subKey))
}

/**
 * 获取帐号附近的几个记录
 * @param {string} accountId
 * @param subKey
 * @return {Promise<unknown>}
 */
export async function getRankNear(accountId: string, subKey?: string) {
  let rank: any = await getAccountRank(accountId)
  if (rank == null) {
    return []
  }
  let total: any = await new RedisClient().zcard(generateRankKey(subKey))
  let start = 0
  let count = 2
  let end = start + count
  if (rank > 0 && rank < total - 1) {
    start = rank - 1
    end = start + count
  } else if (rank == total - 1) {
    start = rank - 2
    end = total
  }
  start = start < 0 ? 0 : start
  end = end > total - 1 ? total - 1 : end
  let datas: any = await getRankList(start, end)
  let results: any[] = []
  for (let i = 0, l = datas.length; i < l; i += 2) {
    results.push({
      rank: start + i / 2,
      accountId: datas[i],
      score: datas[i + 1] << 0,
    })
  }
  return results
}

/**
 * 移除某个榜单
 * @param {string} subKey
 * @return {Promise<void>}
 */
export async function delRankList(subKey?: string) {
  await new RedisClient().del(generateRankKey(subKey))
}

/**
 * 设置在游戏中的状态
 * @param {string} accountId
 * @param seconds
 * @return {Promise<void>}
 */
export async function setGameing(accountId: string, seconds: number) {
  await new RedisClient().setex('gameing_' + accountId, ((Date.now() / 1000) | 0) + '', seconds)
}

export async function setGameEnd(accountId: string) {
  await new RedisClient().expire('gameing_' + accountId, 0)
}
/**
 * 检查是否在游戏中
 * @param {string} accountId
 * @return {Promise<unknown>}
 */
export async function checkGameing(accountId: string) {
  return await new RedisClient().get('gameing_' + accountId)
}
